﻿//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//      The Cy4NET System                      Version 19.10                            //
//      __________________________             _________________________________        //
//                                                                                      //
//      Decentralised Realtime                 Copyright (c) 2012 Reimar Grasbon        //
//      Peer-to-peer Communication             (www.cy4net.org)                         //
//      Protocol                                                                        //
//      _________________________              _________________________________        //
//                                                                                      //
//      [Message]                                                                       //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  This software is subjected to the FreeBSD License. Redistribution and use in source //
//  and binary forms, with or without modification, are permitted provided that the     //
//  following conditions are met:                                                       //
//                                                                                      //
//  1. Redistributions of source code must retain the above copyright notice, this list //
//     of conditions and the following disclaimer.                                      //
//                                                                                      //
//  2. Redistributions in binary form must reproduce the above copyright notice, this   //
//     list of conditions and the following disclaimer in the documentation and/or      //
//     other materials provided with the distribution.                                  //
//                                                                                      //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY //
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED           //
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  //
//  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,    //
//  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  //
//  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR  //
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,   //
//  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)  //
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          //
//  POSSIBILITY OF SUCH DAMAGE.                                                         //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "TypeDefs.h"
#include "Cy4Address.h"

//////////////////////////////////////////////////////////////////////////////////////////
//  Definitionen  ////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
#define CY4MESSAGE(access,   \
                   address,  \
                   location, \
                   count)    { true, access, address, location, count }  // Initialisierer
#define CY4MESSAGE_INVALID   { false }                       // Init.: ungültige Nachricht
#define CY4MESSAGE_CAPACITY  64                              // Nutzlast-Kapazität [Bytes]

//////////////////////////////////////////////////////////////////////////////////////////
//  Typen  ///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
typedef enum                                     // Cy4-Nachricht Zugriffsart:
{   CY4MESSAGE_READ,                             // · Lesenachricht
    CY4MESSAGE_WRITE                             // · Schreibnachricht
} Cy4MessageAccess;

typedef struct                                   // Cy4-Nachricht:
{   bool              valid;                     // · Gültigkeits-Flag ('true' = gültig)
    Cy4MessageAccess  access;                    // · Zugriffsart
    Cy4Address        address;                   // · Cy4-Adresse der Nachricht
    UInt32            location;                  // · Dienstadresse: 0 .. FFFFFEh
    UInt08            count;                     // · Anzahl Nutzlastb.: 1 .. 64 [Bytes]
    UInt08            data[CY4MESSAGE_CAPACITY]; // · Nutzlast
} Cy4Message;

//////////////////////////////////////////////////////////////////////////////////////////
//  Funktionen  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
void              CY4MESSAGE_SetupRead(Cy4Message*, Cy4Address, UInt32, UInt08);
void              CY4MESSAGE_SetupWrite(Cy4Message*, Cy4Address, UInt32, const UInt08*, UInt08);
void              CY4MESSAGE_Invalidate(Cy4Message*);
bool              CY4MESSAGE_IsValid(const Cy4Message*);
Cy4MessageAccess  CY4MESSAGE_Access(const Cy4Message*);
Cy4Address        CY4MESSAGE_Address(const Cy4Message*);
UInt32            CY4MESSAGE_Location(const Cy4Message*);
UInt08            CY4MESSAGE_Count(const Cy4Message*);
const UInt08*     CY4MESSAGE_ConstData(const Cy4Message*);
UInt08*           CY4MESSAGE_Data(Cy4Message*);

//////////////////////////////////////////////////////////////////////////////////////////

